@@ -51,9 +51,9 @@
  *
  * The actors position between the path's end points directly correlates
  * to the #ClutterAlpha's current alpha value driving the behaviour. With
- * the #ClutterAlpha's function set to %CLUTTER_ALPHA_RAMP_INC the actor
+ * the #ClutterAlpha's function set to a linear ramp the actor
  * will follow the path at a constant velocity, but when changing to
- * %CLUTTER_ALPHA_SINE_INC the actor initially accelerates before quickly
+ * a sine wave the actor initially accelerates before quickly
  * decelerating.
  *
  * In order to implement a new behaviour you should subclass #ClutterBehaviour
@@ -325,7 +325,7 @@ clutter_behaviour_class_init (ClutterBehaviourClass *klass)
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (ClutterBehaviourClass, applied),
 		  NULL, NULL,
-		  clutter_marshal_VOID__OBJECT,
+		  _clutter_marshal_VOID__OBJECT,
 		  G_TYPE_NONE, 1,
 		  CLUTTER_TYPE_ACTOR);
   /**
@@ -336,6 +336,7 @@ clutter_behaviour_class_init (ClutterBehaviourClass *klass)
    * The ::removed signal is emitted each time a behaviour is not applied
    * to an actor anymore.
    *
+   * Since: 0.4
    */
   behave_signals[REMOVED] =
     g_signal_new ("removed",
@@ -343,7 +344,7 @@ clutter_behaviour_class_init (ClutterBehaviourClass *klass)
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (ClutterBehaviourClass, removed),
 		  NULL, NULL,
-		  clutter_marshal_VOID__OBJECT,
+		  _clutter_marshal_VOID__OBJECT,
 		  G_TYPE_NONE, 1,
 		  CLUTTER_TYPE_ACTOR);
 
