@@ -20,9 +20,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  *
  * NB: Inspiration for column storage taken from GtkListStore
  */
@@ -110,10 +110,34 @@
  * }
  * </programlisting></informalexample>
  *
- * #ClutterModel is an abstract class. Clutter provides a default model
- * implementation called #ClutterModelDefault which has been optimised
+ * #ClutterModel is an abstract class. Clutter provides a list model
+ * implementation called #ClutterListModel which has been optimised
  * for insertion and look up in sorted lists.
  *
+ * <refsect2 id="ClutterModel-script">
+ *   <title>ClutterModel custom properties for #ClutterScript</title>
+ *   <para>#ClutterModel defines a custom property "columns" for #ClutterScript
+ *   which allows defining the column names and types.</para>
+ *   <example id="ClutterModel-script-column-example">
+ *     <title>Example of the "columns" custom property</title>
+ *     <para>The definition below will create a #ClutterListModel with three
+ *     columns: the first one with name "Name" and containing strings; the
+ *     second one with name "Score" and containing integers; the third one with
+ *     name "Icon" and containing #ClutterTexture<!-- -->s.</para>
+ *     <programlisting>
+ *  {
+ *    "type" : "ClutterListModel",
+ *    "id" : "teams-model",
+ *    "columns" : [
+ *      [ "Name", "gchararray" ],
+ *      [ "Score", "gint" ],
+ *      [ "Icon", "ClutterTexture" ]
+ *    ]
+ *  }
+ *     </programlisting>
+ *   </example>
+ * </refsect2>
+ *
  * #ClutterModel is available since Clutter 0.6
  */
 
@@ -132,8 +156,14 @@
 #include "clutter-marshal.h"
 #include "clutter-private.h"
 #include "clutter-debug.h"
+#include "clutter-scriptable.h"
 
-G_DEFINE_ABSTRACT_TYPE (ClutterModel, clutter_model, G_TYPE_OBJECT);
+static void clutter_scriptable_iface_init (ClutterScriptableIface *iface);
+
+G_DEFINE_ABSTRACT_TYPE_WITH_CODE
+        (ClutterModel, clutter_model, G_TYPE_OBJECT,
+         G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_SCRIPTABLE,
+                                clutter_scriptable_iface_init));
 
 enum
 {
@@ -163,6 +193,15 @@ struct _ClutterModelPrivate
 {
   GType                  *column_types;
   gchar                 **column_names;
+
+  /* we use an integer here because we want to be able to use -1 as a
+   * guard value, to allow calling set_names() and set_types() from
+   * sub-classes of ClutterModel. see bug:
+   *
+   *   http://bugzilla.openedhand.com/show_bug.cgi?id=2032
+   *
+   * for a reference.
+   */
   gint                    n_columns; 
 
   ClutterModelFilterFunc  filter_func;
@@ -205,7 +244,12 @@ clutter_model_real_get_column_name (ClutterModel *model,
 static guint
 clutter_model_real_get_n_columns (ClutterModel *model)
 {
-  return model->priv->n_columns;
+  ClutterModelPrivate *priv = model->priv;
+
+  if (priv->n_columns < 0)
+    return 0;
+
+  return priv->n_columns;
 }
 
 static void 
@@ -222,13 +266,16 @@ clutter_model_finalize (GObject *object)
 
   g_free (priv->column_types);
 
-  /* the column_names vector might have holes in it, so we need to
-   * use the columns number to clear up everything
-   */
-  for (i = 0; i < priv->n_columns; i++)
-    g_free (priv->column_names[i]);
+  if (priv->column_names != NULL)
+    {
+      /* the column_names vector might have holes in it, so we need
+       * to use the columns number to clear up everything
+       */
+      for (i = 0; i < priv->n_columns; i++)
+        g_free (priv->column_names[i]);
 
-  g_free (priv->column_names);
+      g_free (priv->column_names);
+    }
 
   G_OBJECT_CLASS (clutter_model_parent_class)->finalize (object);
 }
@@ -302,7 +349,7 @@ clutter_model_class_init (ClutterModelClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterModelClass, row_added),
                   NULL, NULL,
-                  clutter_marshal_VOID__OBJECT,
+                  _clutter_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1,
                   CLUTTER_TYPE_MODEL_ITER);
    /**
@@ -322,7 +369,7 @@ clutter_model_class_init (ClutterModelClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterModelClass, row_removed),
                   NULL, NULL,
-                  clutter_marshal_VOID__OBJECT,
+                  _clutter_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1,
                   CLUTTER_TYPE_MODEL_ITER);
    /**
@@ -342,7 +389,7 @@ clutter_model_class_init (ClutterModelClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterModelClass, row_changed),
                   NULL, NULL,
-                  clutter_marshal_VOID__OBJECT,
+                  _clutter_marshal_VOID__OBJECT,
                   G_TYPE_NONE, 1,
                   CLUTTER_TYPE_MODEL_ITER);
   /**
@@ -359,7 +406,7 @@ clutter_model_class_init (ClutterModelClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterModelClass, sort_changed),
                   NULL, NULL,
-                  clutter_marshal_VOID__VOID,
+                  _clutter_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
    /**
    * ClutterModel::filter-changed:
@@ -375,7 +422,7 @@ clutter_model_class_init (ClutterModelClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterModelClass, filter_changed),
                   NULL, NULL,
-                  clutter_marshal_VOID__VOID,
+                  _clutter_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 }
 
@@ -442,6 +489,103 @@ clutter_model_check_type (GType gtype)
 }
 
 
+typedef struct {
+    gchar *name;
+    GType  type;
+} ColumnInfo;
+
+static gboolean
+clutter_model_parse_custom_node (ClutterScriptable *scriptable,
+                                 ClutterScript     *script,
+                                 GValue            *value,
+                                 const gchar       *name,
+                                 JsonNode          *node)
+{
+  GSList *columns = NULL;
+  GList *elements, *l;
+
+  if (strcmp (name, "columns") != 0)
+    return FALSE;
+
+  if (JSON_NODE_TYPE (node) != JSON_NODE_ARRAY)
+    return FALSE;
+
+  elements = json_array_get_elements (json_node_get_array (node));
+
+  for (l = elements; l != NULL; l = l->next)
+    {
+      JsonNode *child_node = l->data;
+      JsonArray *array = json_node_get_array (child_node);
+      ColumnInfo *cinfo;
+      const gchar *column_name;
+      const gchar *type_name;
+
+      if (JSON_NODE_TYPE (node) != JSON_NODE_ARRAY ||
+          json_array_get_length (array) != 2)
+        {
+          g_warning ("A column must be an array of "
+                     "[\"column-name\", \"GType-name\"] pairs");
+          return FALSE;
+        }
+
+      column_name = json_array_get_string_element (array, 0);
+      type_name = json_array_get_string_element (array, 1);
+
+      cinfo = g_slice_new0 (ColumnInfo);
+      cinfo->name = g_strdup (column_name);
+      cinfo->type = clutter_script_get_type_from_name (script, type_name);
+
+      columns = g_slist_prepend (columns, cinfo);
+    }
+
+  g_list_free (elements);
+
+  g_value_init (value, G_TYPE_POINTER);
+  g_value_set_pointer (value, g_slist_reverse (columns));
+
+  return TRUE;
+}
+
+static void
+clutter_model_set_custom_property (ClutterScriptable *scriptable,
+                                   ClutterScript     *script,
+                                   const gchar       *name,
+                                   const GValue      *value)
+{
+  if (strcmp (name, "columns") == 0)
+    {
+      ClutterModel *model = CLUTTER_MODEL (scriptable);
+      GSList *columns, *l;
+      guint n_columns;
+      gint i;
+
+      columns = g_value_get_pointer (value);
+      n_columns = g_slist_length (columns);
+
+      clutter_model_set_n_columns (model, n_columns, TRUE, TRUE);
+
+      for (i = 0, l = columns; l != NULL; l = l->next, i++)
+        {
+          ColumnInfo *cinfo = l->data;
+
+          clutter_model_set_column_name (model, i, cinfo->name);
+          clutter_model_set_column_type (model, i, cinfo->type);
+
+          g_free (cinfo->name);
+          g_slice_free (ColumnInfo, cinfo);
+        }
+
+      g_slist_free (columns);
+    }
+}
+
+
+static void
+clutter_scriptable_iface_init (ClutterScriptableIface *iface)
+{
+  iface->parse_custom_node = clutter_model_parse_custom_node;
+  iface->set_custom_property = clutter_model_set_custom_property;
+}
 
 /**
  * clutter_model_resort:
@@ -560,10 +704,6 @@ clutter_model_set_n_columns (ClutterModel *model,
                              gboolean      set_names)
 {
   ClutterModelPrivate *priv = model->priv;
-  gboolean types_set, names_set;
-
-  types_set = (priv->column_types != NULL);
-  names_set = (priv->column_names != NULL);
 
   if (priv->n_columns > 0 && priv->n_columns != n_columns)
     return;
@@ -787,12 +927,10 @@ void
 clutter_model_append (ClutterModel *model,
                       ...)
 {
-  ClutterModelPrivate *priv;
   ClutterModelIter *iter;
   va_list args;
 
   g_return_if_fail (CLUTTER_IS_MODEL (model));
-  priv = model->priv;
 
   iter = CLUTTER_MODEL_GET_CLASS (model)->insert_row (model, -1);
   g_assert (CLUTTER_IS_MODEL_ITER (iter));
@@ -1081,12 +1219,10 @@ G_CONST_RETURN gchar *
 clutter_model_get_column_name (ClutterModel *model,
                                guint         column)
 {
-  ClutterModelPrivate *priv;
   ClutterModelClass *klass;
 
   g_return_val_if_fail (CLUTTER_IS_MODEL (model), NULL);
 
-  priv = model->priv;
   if (column < 0 || column >= clutter_model_get_n_columns (model))
     {
       g_warning ("%s: Invalid column id value %d\n", G_STRLOC, column);
@@ -1115,12 +1251,10 @@ GType
 clutter_model_get_column_type (ClutterModel *model,
                                guint         column)
 {
-  ClutterModelPrivate *priv;
   ClutterModelClass *klass;
 
   g_return_val_if_fail (CLUTTER_IS_MODEL (model), G_TYPE_INVALID);
 
-  priv = model->priv;
   if (column < 0 || column >= clutter_model_get_n_columns (model))
     {
       g_warning ("%s: Invalid column id value %d\n", G_STRLOC, column);
